require "gooi"
require "animManager"
require "colorManager"
require "AnAL"
require "pixel"

function love.load()
	gr = love.graphics
	kb = love.keyboard
	mo = love.mouse
	im = love.image
	to = love.touch

	save_dir = love.filesystem.getSaveDirectory()
	print("GÖÖi save directory: "..save_dir)

	layout.padding = 1 -- default  = 3

	fontDir = "/fonts/"
	imgDir = "/imgs/"

	width = gr.getWidth()
	height = gr.getHeight()

	wCanvas = 100
	hCanvas = wCanvas * height / width
	scale = width / wCanvas

	imgBg = gr.newImage(imgDir.."bg.png")
	imgPaletteSmall = gr.newImage(imgDir.."paletteSmall.png")
	imgColorPickerSmall = gr.newImage(imgDir.."colorPickerSmall.png")
	imgTransparent = gr.newImage(imgDir.."transparent.png")
	imgPlay = gr.newImage(imgDir.."play.png")
	imgStop = gr.newImage(imgDir.."stop.png")
	imgCenterBlinker = gr.newImage(imgDir.."centerBlinker.png")
	imgInnerBlinker = gr.newImage(imgDir.."innerBlinker.png")
	imgInnerBlinkerUp = gr.newImage(imgDir.."innerBlinkerUp.png")
	imgOuterBlinker = gr.newImage(imgDir.."outerBlinker.png")
	imgOuterBlinkerUp = gr.newImage(imgDir.."outerBlinkerUp.png")
	imgBgCanvas = gr.newImage(imgDir.."bgCanvas.png")
	imgMoveFrame = gr.newImage(imgDir.."moveFrame.png")
	imgLeft = gr.newImage(imgDir.."left.png")
	imgRight = gr.newImage(imgDir.."right.png")
	imgNew = gr.newImage(imgDir.."new.png")
	imgOpen = gr.newImage(imgDir.."open.png")
	imgSave = gr.newImage(imgDir.."save.png")
	imgSaveAs = gr.newImage(imgDir.."saveAs.png")
	imgMatrixPalette = gr.newImage(imgDir.."matrixPalette.png")
	imgMirrorX = gr.newImage(imgDir.."mirrorX.png")
	imgMirrorY = gr.newImage(imgDir.."mirrorY.png")
	imgEraser = gr.newImage(imgDir.."eraser.png");

	--[[
	imgBg:setFilter("nearest", "nearest")
	imgLeft:setFilter("nearest", "nearest")
	imgRight:setFilter("nearest", "nearest")
	]]
	imgBgCanvas:setWrap("repeat", "repeat")

	idPaletteSmall = im.newImageData(imgDir.."paletteSmall.png")
	
	--[[
	for i=1,idPaletteSmall:getHeight() do
		for j=1,idPaletteSmall:getWidth() do
			local r, g, b = idPaletteSmall:getPixel(j - 1, i - 1)
			print(r..", "..g..", "..b)
		end
	end
	]]

	canvas = gr.newCanvas(wCanvas, hCanvas)
	canvas:setFilter("nearest", "nearest")
	gooi.setCanvas(canvas)

	style = {
		font = gr.newFont(fontDir.."04B_03__.TTF", 8)
	}
	gooi.setStyle(style)

	dispPalette = 1
	dispControls = 37

	joyPalette = gooi.newJoy({
		x = dispPalette,
		y = dispPalette,
		size = 34}):bg({0, 0, 0, 0})
	joyPalette:noSpring()
	joyPalette:anyPoint()
	joyPalette:setImage(imgTransparent):noScaling()
	joyPalette.rStick = 4

	joyMove = gooi.newJoy({
		x = 58,
		y = hCanvas - 42,
		size = 40
	}):bg({0, 0, 0, 0})
	--:setImage(imgMoveFrame)
	:setImage(imgTransparent)
	:noScaling()
	
	brush = {
		color = {idPaletteSmall:getPixel(14, 13)}
	}

	-- Edit GUI:
	panelEdit = gooi.newPanel(36, 0, 64, 36, "grid 3x4")
	panelEdit:setColspan(1, 2, 2)
	panelEdit:setRowspan(1, 2, 2)
	panelEdit:add(
		gooi.newButton({text = "", icon = imgNew}):bg("#00880088"),
		gooi.newButton({text = "", icon = imgPlay}):bg("#00880088"):onRelease(function(c)
			if c.playing then
				c.playing = false
				c:bg("#00880088"):setIcon(imgPlay)
			else
				c.playing = true
				c:bg("#BB000088"):setIcon(imgStop)
			end
		end),
		gooi.newButton({text = "", icon = imgSave}):bg("#BB000088"),
		gooi.newButton({text = "", icon = imgOpen}):bg("#00880088"),
		gooi.newButton({text = "", icon = imgSaveAs}):bg("#BB000088"),
		gooi.newButton({text = "", icon = imgMatrixPalette}),
		gooi.newButton({text = "", icon = imgMirrorX}):bg("#88888888"),
		gooi.newButton({text = "", icon = imgMirrorY}):bg("#88888888"),
		gooi.newButton({text = "", icon = imgEraser}):bg("#FFFFFF")
	)--:setOpaque(true)
	
	currentFrameIndex = 1
	brightBlinker = 255
	alphaMove = 0

	btnMoveLeft = gooi.newButton():bg({12, 183, 242, alphaMove}):onRelease(function(c)
		alphaMove = 170
	end)
	btnMoveRight = gooi.newButton():bg({12, 183, 242, alphaMove}):onRelease(function(c)
		alphaMove = 170
	end)

	panelMove = gooi.newPanel(1, 37, wCanvas - 1, 16, "grid 1x4")--:setOpaque(true)
	panelMove:add(btnMoveLeft, "1,1")
	panelMove:add(btnMoveRight, "1,4")
	--panelEdit:setOpaque(true)

	gr.setLineWidth(1)
	gr.setLineStyle("rough")

	wFrame, hFrame = 8, 8

	pS = 8

	font = gooi.getFont()
	gr.setFont(font)


	-- Zooming:
	currentZoom = 1
	applyZoom(currentZoom)
	zooming = false
	initZoomDistance = 0
	zoomTouch1, zoomTouch2 = nil, nil
	setNewAnim(10, wFrame, hFrame)
end







----------------------------------------
----------------------------------------
-- UPDATE
----------------------------------------
----------------------------------------

function love.update(dt)
	gooi.update(dt)

	-- update brush color:
	brush.color = {idPaletteSmall:getPixel(
		joyPalette.xStick - dispPalette,
		joyPalette.yStick - dispPalette)}
	
	brightBlinker = brightBlinker - dt * 500
	if brightBlinker <= 0 then
		brightBlinker = 255
	end

	if false then--joyMove.touch then
		xFrame = xFrame - joyMove:xValue() * 120 * dt
		yFrame = yFrame - joyMove:yValue() * 120 * dt
	end

	-- alpha of move btns:
	alphaMove = alphaMove - 100 * dt
	if alphaMove <= 0 then
		alphaMove = 0
	end

	btnMoveRight.bgColor[4] = alphaMove
	btnMoveLeft.bgColor[4] = alphaMove

	--[[
	]]
end







----------------------------------------
----------------------------------------
-- DRAW
----------------------------------------
----------------------------------------

function love.draw()
	-- Do canvas stuff:
	gr.setCanvas(canvas)
	gr.clear()

	gr.setLineWidth(1)
	gr.setLineStyle("rough")

	gr.setColor(255, 255, 255)
	gr.draw(imgBg,
		0, hCanvas - imgBg:getHeight(),
		0, 1, 1
	)

	-- Frames:
	drawCurrentFrame()
	
	-- Palette:
	gr.setColor(255, 255, 255)
	gr.draw(imgPaletteSmall, joyPalette.x, joyPalette.y)

	-- Color blinker:
	local imgBlinker, scaleBX, scaleBY = getImageBlinker(brush.color)

	gr.setColor(brightBlinker, brightBlinker, brightBlinker)
	gr.draw(imgBlinker,
		joyPalette.x + joyPalette.w / 2,
		joyPalette.y + joyPalette.h / 2,
		0, scaleBX, scaleBY,
		imgBlinker:getWidth() / 2,
		imgBlinker:getHeight() / 2)

	--GÖÖi:
	gr.setColor(255, 255, 255)
	gooi.draw()
	gr.setColor(255, 255, 255)

	-- Move arrows:
	gr.draw(imgLeft,
		math.floor(btnMoveLeft.x + btnMoveLeft.w / 2),
		math.floor(btnMoveLeft.y + btnMoveLeft.h / 2),
		0, 1, 1,
		math.floor(imgLeft:getWidth() / 2),
		math.floor(imgLeft:getHeight() / 2))

	gr.draw(imgRight,
		math.floor(btnMoveRight.x + btnMoveRight.w / 2),
		math.floor(btnMoveRight.y + btnMoveRight.h / 2),
		0, 1, 1,
		math.floor(imgRight:getWidth() / 2),
		math.floor(imgRight:getHeight() / 2))

	--[[
	gr.setColor(255, 0, 0)
	gr.rectangle("fill", joyPalette.xStick, joyPalette.yStick, 1, 1)
	gr.setColor(255, 255, 255)
	]]

	gr.setColor(0, 0, 0, 70)
	gr.rectangle("fill", 0, math.floor(hCanvas - font:getHeight()) - 1, 32, 10)
	gr.setColor(255, 255, 255)
	gr.print("FPS: "..love.timer.getFPS(),
		1, math.floor(hCanvas - font:getHeight()) - 1)

	gr.setCanvas()
	gr.draw(canvas, 0, 0, 0, scale, scale)
end









----------------------------------------
----------------------------------------
-- STUFF
----------------------------------------
----------------------------------------
function applyZoom(zoom)
	realWFrame = pS * wFrame * zoom
	realHFrame = pS * hFrame * zoom
	xFrame = math.floor(wCanvas / 2 - realWFrame / 2)
	yFrame = math.floor(hCanvas / 2 - realHFrame / 2 ) + 18
	quadBgCanvas = gr.newQuad(0, 0,
		realWFrame - 1,
		realHFrame - 1,
		imgBgCanvas:getDimensions())
end

function insideCanvas(x, y)
	return not (
		x / scale <= xFrame or
		x / scale >= xFrame + realWFrame or
		y / scale <= yFrame or
		y / scale >= yFrame + realHFrame
	)
end

function setPixel(x, y, c, fromUpdate)
	-- Transform to the virtual pixels on the current frame:
	if insideCanvas(x, y) then
		local fixedX, fixedY = getFixedPosition(x, y)
		--print("fixedX, fixedX = "..fixedX..", "..fixedY)
		local cF = frames[currentFrameIndex]
		local fW, fH = #cF[1], #cF
		--print(fixedX, fixedY)

		--flood(fixedX, fixedY, cF, cF[fixedY][fixedX].color, brush.color)
		local row = cF[fixedY]
		local cell = row[fixedX]:setColor(c)

		--[[
		-- Draw a line if it's the case (Bresenham's Line Algorithm):
		if lineAnchor and shiftDown() then
			bresenham(lineAnchor[1], lineAnchor[2], fixedX, fixedY, fW, fH, cF,c)
		end

		-- Mirrors:
		drawMirrors(fixedX, fixedY, fW, fH, cF, c)
		
		-- Autoanimation:
		drawAutoAnim(fixedX, fixedY, fW, fH, cF, c)

		notifyNotSaved()
		]]
	end
end

function getFixedPosition(xt, yt)
	local x = math.ceil((xt / scale - xFrame) / pS)
	local y = math.ceil((yt / scale - yFrame) / pS)
	return x, y
end

function getImageBlinker(c)
	local sx, sy = 1, 1
	local img = imgTransparent

	if isColor(c, 222, 238, 214) then img = imgCenterBlinker  end
	if isColor(c, 218, 212, 94) then  img = imgInnerBlinkerUp end
	if isColor(c, 210, 170, 153) then img = imgInnerBlinker   end
	if isColor(c, 117, 113, 97) then  img = imgOuterBlinker   end
	if isColor(c, 133, 76, 48) then   img = imgOuterBlinkerUp end
	if isColor(c, 78, 74, 78) then    img = imgOuterBlinker    sx = -1  end
	if isColor(c, 89, 125, 206) then  img = imgInnerBlinker;   sx = -1  end
	if isColor(c, 109, 194, 202) then img = imgInnerBlinkerUp; sx = -1  end
	if isColor(c, 48, 52, 109) then   img = imgOuterBlinkerUp; sx = -1  end
	if isColor(c, 203, 154, 219) then img = imgOuterBlinker;   sy = -1  end
	if isColor(c, 109, 170, 44) then  img = imgInnerBlinker;   sy = -1  end
	if isColor(c, 210, 125, 44) then  img = imgInnerBlinkerUp  sy = -1  end
	if isColor(c, 20, 12, 28) then    img = imgOuterBlinkerUp; sy = -1  end
	if isColor(c, 133, 149, 161) then img = imgInnerBlinker    sx, sy = -1, -1 end
	if isColor(c, 52, 101, 36) then   img = imgOuterBlinker    sx, sy = -1, -1 end
	if isColor(c, 68, 36, 52) then    img = imgOuterBlinkerUp; sx, sy = -1, -1 end
	if isColor(c, 208, 70, 72) then   img = imgInnerBlinkerUp; sx, sy = -1, -1 end

	return img, sx, sy
end

function isColor(c, r, g, b)
	return c[1] == r and c[2] == g and c[3] == b
end

function drawCurrentFrame()
	gr.setColor(0, 0, 0, 63)
	gr.rectangle("fill", xFrame + 1, yFrame + 1, realWFrame, realHFrame)
	gr.setColor(255, 255, 255)
	gr.draw(imgBgCanvas, quadBgCanvas, xFrame, yFrame)

	if playing then
		--analAnim:draw(xCanvas, yCanvas, 0, pS, pS)
	else
		local f = frames[currentFrameIndex]
		for i = 1, #f do
			for j = 1, #f[i] do
				local n, m = 0, 0
				if i == #f then m = 1 end
				if j == #f[i] then n = 1 end
				local p = f[i][j]
				gr.setColor(p.color)
				gr.rectangle("fill",
					(xFrame) + (j - 1) * pS,
					(yFrame) + (i - 1) * pS, pS - n, pS - m)
			end
		end
	end

	-- Grid:
	gr.setColor(0, 0, 0, 80)
	for i = 1, wFrame - 1 do
		gr.line(
			(xFrame + i * pS * currentZoom),
			(yFrame),
			(xFrame + i * pS * currentZoom),
			(yFrame + realHFrame - 1)
		)
	end

	for i = 1, hFrame - 1 do
		gr.line(
			(xFrame),
			(yFrame + i * pS * currentZoom),
			(xFrame + realWFrame - 1),
			(yFrame + i * pS * currentZoom)
		)
	end
end

function love.touchpressed(id, x, y)
	gooi.pressed(id, x, y)

	local touches = to.getTouches()
	touchNbr = #touches
	-- Check zoom:
	if touchNbr == 2 then
		local x1, y1 = to.getPosition(touches[1])
		local x2, y2 = to.getPosition(touches[2])

		x1 = x1 / scale
		x2 = x2 / scale
		y1 = y1 / scale
		y2 = y2 / scale

		if y1 > dispControls and y2 > dispControls then
			local dx = math.abs(x1 - x2)
			local dy = math.abs(y1 - y2)

			initZoomDistance = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))

			zoomTouch1 = {x1, y1}
			zoomTouch2 = {x2, y2}

			zooming = true
		end
	elseif touchNbr == 1 then

	end
end

function love.touchmoved(id, x, y)
	gooi.moved(id, x, y)

	if zoomTouch1 and zoomTouch2 then
		local touches = to.getTouches()
		if #touches == 2 and zooming then
			local x1, y1 = to.getPosition(touches[1])
			local x2, y2 = to.getPosition(touches[2])

			x1 = x1 / scale
			x2 = x2 / scale
			y1 = y1 / scale
			y2 = y2 / scale

			local dx = math.abs(x1 - x2)
			local dy = math.abs(y1 - y2)

			currentZoomDistance = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))
			currentZoom = currentZoomDistance / initZoomDistance
			applyZoom(currentZoom)
		end
	end
end

function love.touchreleased(id, x, y)
	gooi.released(id, x, y)

	local touches = to.getTouches()
	local touchNbr = #touches

	print("** touchNbr: "..touchNbr)

	if zooming then
		if touchNbr == 0 then
			zoomTouch1, zoomTouch2 = nil, nil
			pS = pS * currentZoom
			currentZoom = 1
			zooming = false
		end
	else
		if touchNbr == 0 then
			setPixel(x, y, brush.color)
		end
	end
end

function setNewAnim(n, theW, theH)
	currentFrameIndex = 1
	frames = animManager.createFrames(n, theW, theH)
end

--[[
function love.mousereleased(x, y, button) gooi.released() end
function love.mousepressed(x, y, button)  gooi.pressed() end
]]

function love.textinput(text)
	gooi.textinput(text)
end

function love.keypressed(key)
	gooi.keypressed(key)

	if key == "escape" then
		quit()
	end
end

function quit()
	love.event.quit()
end

function r()
	return love.math.random(0, 255)
end
